@@ -1,12 +1,7 @@
+import json
 import os
 
 import docker.client
-from dagster_celery.config import DEFAULT_CONFIG, dict_wrapper
-from dagster_celery.core_execution_loop import DELEGATE_MARKER, core_celery_execution_loop
-from dagster_celery.defaults import broker_url, result_backend
-from dagster_celery.executor import CELERY_CONFIG
-from dagster_graphql.client.mutations import handle_execute_plan_result, handle_execution_errors
-
 from dagster import (
     DagsterInstance,
     EventMetadataEntry,
@@ -15,16 +10,18 @@ from dagster import (
     StringSource,
     check,
     executor,
-    seven,
+    multiple_process_executor_requirements,
 )
-from dagster.core.definitions.executor import check_cross_process_constraints
+from dagster.cli.api import ExecuteStepArgs
 from dagster.core.events import EngineEventData
-from dagster.core.execution.retries import Retries
-from dagster.core.host_representation.handle import IN_PROCESS_NAME
-from dagster.core.instance import InstanceRef
-from dagster.serdes import serialize_dagster_namedtuple
-from dagster.seven import JSONDecodeError
+from dagster.core.execution.retries import RetryMode
+from dagster.core.storage.pipeline_run import PipelineRun
+from dagster.serdes import pack_value, serialize_dagster_namedtuple, unpack_value
 from dagster.utils import merge_dicts
+from dagster_celery.config import DEFAULT_CONFIG, dict_wrapper
+from dagster_celery.core_execution_loop import DELEGATE_MARKER, core_celery_execution_loop
+from dagster_celery.defaults import broker_url, result_backend
+from dagster_celery.executor import CELERY_CONFIG
 
 CELERY_DOCKER_CONFIG_KEY = "celery-docker"
 
@@ -35,7 +32,7 @@ def celery_docker_config():
             {
                 "image": Field(
                     StringSource,
-                    is_required=True,
+                    is_required=False,
                     description="The docker image to be used for step execution.",
                 ),
                 "registry": Field(
@@ -52,23 +49,26 @@ def celery_docker_config():
                     is_required=False,
                     description="The list of environment variables names to forward from the celery worker in to the docker container",
                 ),
+                "network": Field(
+                    str,
+                    is_required=False,
+                    description="Name of the network this container will be connected to at creation time",
+                ),
             },
             is_required=True,
             description="The configuration for interacting with docker in the celery worker.",
         ),
-        "repo_location_name": Field(
-            StringSource,
-            is_required=False,
-            default_value=IN_PROCESS_NAME,
-            description="[temporary workaround] The repository location name to use for execution.",
-        ),
     }
 
     cfg = merge_dicts(CELERY_CONFIG, additional_config)
     return cfg
 
 
-@executor(name=CELERY_DOCKER_CONFIG_KEY, config_schema=celery_docker_config())
+@executor(
+    name=CELERY_DOCKER_CONFIG_KEY,
+    config_schema=celery_docker_config(),
+    requirements=multiple_process_executor_requirements(),
+)
 def celery_docker_executor(init_context):
     """Celery-based executor which launches tasks in docker containers.
 
@@ -133,7 +133,6 @@ def celery_docker_executor(init_context):
     In deployments where the celery_k8s_job_executor is used all appropriate celery and dagster_celery
     commands must be invoked with the `-A dagster_celery_docker.app` argument.
     """
-    check_cross_process_constraints(init_context)
 
     exc_cfg = init_context.executor_config
 
@@ -142,9 +141,8 @@ def celery_docker_executor(init_context):
         backend=exc_cfg.get("backend"),
         config_source=exc_cfg.get("config_source"),
         include=exc_cfg.get("include"),
-        retries=Retries.from_config(exc_cfg.get("retries")),
+        retries=RetryMode.from_config(exc_cfg.get("retries")),
         docker_config=exc_cfg.get("docker"),
-        repo_location_name=exc_cfg.get("repo_location_name"),
     )
 
 
@@ -157,9 +155,8 @@ class CeleryDockerExecutor(Executor):
         backend=None,
         include=None,
         config_source=None,
-        repo_location_name=None,
     ):
-        self._retries = check.inst_param(retries, "retries", Retries)
+        self._retries = check.inst_param(retries, "retries", RetryMode)
         self.broker = check.opt_str_param(broker, "broker", default=broker_url)
         self.backend = check.opt_str_param(backend, "backend", default=result_backend)
         self.include = check.opt_list_param(include, "include", of_type=str)
@@ -167,7 +164,6 @@ class CeleryDockerExecutor(Executor):
             dict(DEFAULT_CONFIG, **check.opt_dict_param(config_source, "config_source"))
         )
         self.docker_config = check.dict_param(docker_config, "docker_config")
-        self.repo_location_name = check.str_param(repo_location_name, "repo_location_name")
 
     @property
     def retries(self):
@@ -189,19 +185,19 @@ class CeleryDockerExecutor(Executor):
         }
 
 
-def _submit_task_docker(app, pipeline_context, step, queue, priority):
+def _submit_task_docker(app, pipeline_context, step, queue, priority, known_state):
+    execute_step_args = ExecuteStepArgs(
+        pipeline_origin=pipeline_context.reconstructable_pipeline.get_python_origin(),
+        pipeline_run_id=pipeline_context.pipeline_run.run_id,
+        step_keys_to_execute=[step.key],
+        instance_ref=pipeline_context.instance.get_ref(),
+        retry_mode=pipeline_context.executor.retries.for_inner_plan(),
+        known_state=known_state,
+    )
+
     task = create_docker_task(app)
-
-    recon_repo = pipeline_context.pipeline.get_reconstructable_repository()
-
     task_signature = task.si(
-        instance_ref_dict=pipeline_context.instance.get_ref().to_dict(),
-        step_keys=[step.key],
-        run_config=pipeline_context.pipeline_run.run_config,
-        mode=pipeline_context.pipeline_run.mode,
-        repo_name=recon_repo.get_definition().name,
-        repo_location_name=pipeline_context.executor.repo_location_name,
-        run_id=pipeline_context.pipeline_run.run_id,
+        execute_step_args_packed=pack_value(execute_step_args),
         docker_config=pipeline_context.executor.docker_config,
     )
     return task_signature.apply_async(
@@ -215,45 +211,42 @@ def create_docker_task(celery_app, **task_kwargs):
     @celery_app.task(bind=True, name="execute_step_docker", **task_kwargs)
     def _execute_step_docker(
         self,
-        instance_ref_dict,
-        step_keys,
-        run_config,
-        mode,
-        repo_name,
-        repo_location_name,
-        run_id,
+        execute_step_args_packed,
         docker_config,
     ):
-        """Run step execution in a Docker container.
-        """
-        instance_ref = InstanceRef.from_dict(instance_ref_dict)
-        instance = DagsterInstance.from_ref(instance_ref)
-        pipeline_run = instance.get_run_by_id(run_id)
-        check.invariant(pipeline_run, "Could not load run {}".format(run_id))
-
-        step_keys_str = ", ".join(step_keys)
-
-        variables = {
-            "executionParams": {
-                "runConfigData": run_config,
-                "mode": mode,
-                "selector": {
-                    "repositoryLocationName": repo_location_name,
-                    "repositoryName": repo_name,
-                    "pipelineName": pipeline_run.pipeline_name,
-                    "solidSelection": list(pipeline_run.solids_to_execute)
-                    if pipeline_run.solids_to_execute
-                    else None,
-                },
-                "executionMetadata": {"runId": run_id},
-                "stepKeys": step_keys,
-            }
-        }
-
-        command = "dagster-graphql -v '{variables}' -p executePlan".format(
-            variables=seven.json.dumps(variables)
+        """Run step execution in a Docker container."""
+        execute_step_args = unpack_value(
+            check.dict_param(
+                execute_step_args_packed,
+                "execute_step_args_packed",
+            )
         )
-        docker_image = docker_config["image"]
+        check.inst_param(execute_step_args, "execute_step_args", ExecuteStepArgs)
+
+        check.dict_param(docker_config, "docker_config")
+
+        instance = DagsterInstance.from_ref(execute_step_args.instance_ref)
+        pipeline_run = instance.get_run_by_id(execute_step_args.pipeline_run_id)
+        check.inst(
+            pipeline_run,
+            PipelineRun,
+            "Could not load run {}".format(execute_step_args.pipeline_run_id),
+        )
+        step_keys_str = ", ".join(execute_step_args.step_keys_to_execute)
+
+        input_json = serialize_dagster_namedtuple(execute_step_args)
+
+        command = "dagster api execute_step {}".format(json.dumps(input_json))
+
+        docker_image = (
+            docker_config["image"]
+            if docker_config.get("image")
+            else execute_step_args.pipeline_origin.repository_origin.container_image
+        )
+
+        if not docker_image:
+            raise Exception("No docker image specified by either the pipeline or the repository")
+
         client = docker.client.from_env()
 
         if docker_config.get("registry"):
@@ -276,10 +269,10 @@ def create_docker_task(celery_app, **task_kwargs):
                 marker_end=DELEGATE_MARKER,
             ),
             CeleryDockerExecutor,
-            step_key=step_keys[0],
+            step_key=execute_step_args.step_keys_to_execute[0],
         )
 
-        events = [engine_event]
+        serialized_events = [serialize_dagster_namedtuple(engine_event)]
 
         docker_env = {}
         if docker_config.get("env_vars"):
@@ -293,9 +286,10 @@ def create_docker_task(celery_app, **task_kwargs):
                 auto_remove=True,
                 # pass through this worker's environment for things like AWS creds etc.
                 environment=docker_env,
+                network=docker_config.get("network", None),
             )
-            res = seven.json.loads(docker_response)
 
+            res = docker_response.decode("utf-8")
         except docker.errors.ContainerError as err:
             instance.report_engine_event(
                 "Failed to run steps {} in Docker container {}".format(step_keys_str, docker_image),
@@ -307,34 +301,15 @@ def create_docker_task(celery_app, **task_kwargs):
                     ],
                 ),
                 CeleryDockerExecutor,
-                step_key=step_keys[0],
+                step_key=execute_step_args.step_keys_to_execute[0],
             )
             raise
-
-        except JSONDecodeError:
-            instance.report_engine_event(
-                "Failed to parse response for steps {} from Docker container {}".format(
-                    step_keys_str, docker_image
-                ),
-                pipeline_run,
-                EngineEventData(
-                    [
-                        EventMetadataEntry.text(docker_image, "Job image"),
-                        EventMetadataEntry.text(docker_response, "Docker Response"),
-                    ],
-                ),
-                CeleryDockerExecutor,
-                step_key=step_keys[0],
-            )
-            raise
-
         else:
-            handle_execution_errors(res, "executePlan")
-            step_events = handle_execute_plan_result(res)
+            if res is None:
+                raise Exception("No response from execute_step in CeleryDockerExecutor")
 
-        events += step_events
+            serialized_events += [event for event in res.split("\n") if event]
 
-        serialized_events = [serialize_dagster_namedtuple(event) for event in events]
         return serialized_events
 
     return _execute_step_docker
